#! /bin/gawk -f

# Script for generating atom feed entries from exported news entries.

# Script arguments:
# - baseurl :: URL of your website, required

# Example call:
# ./atomize-news-entry.awk -v baseurl=www.example.com ../build/news-exported/2023-05-03-Wed-12-00-foo.org

BEGIN {
  if (!baseurl) {
    print "Required argument 'baseurl' not provided" > "/dev/stderr"
    exit 1
  }
  print("<entry>");
}

# Interpret the first heading as the title
/<h.>.*<\/h.>/ && !PrintedMetadata {
  # Print title
  print gensub(/<h.>(.*)<\/h.>/, "<title>\\1</title>", "g")
  # Print time
  # We obfuscate hour and minute info in news entries, but looks like RFC4287
  # requires them, so retrieve them from filename.
  "basename " FILENAME | getline basename
  time = gensub(/([0-9]{4})-([0-9]{2})-([0-9]{2})-[A-Za-z]*-([0-9]{2})-([0-9]{2})-.*/,
                "\\1-\\2-\\3T\\4:\\5:00Z", "g", basename)
  printf "<updated>%s</updated>\n", time
  # Print ID - these need to be reproducible so generate them from md5 hashes
  "echo -n " basename " | md5sum | awk '{print $1}'" | getline md5sum
  printf "<id>urn:uuid:%s</id>\n", md5_to_uuid(md5sum)
  # Begin content
  printf "<content type=\"html\" xml:lang=\"en\" xml:base=\"%s\">\n", baseurl
  printf "<![CDATA["

  PrintedMetadata = 1
  next;
}

/<time datetime=.*>.*<\/time>/ {
  # Skip printing the time element in content as it's included in metadata
  next;
}

// {
  if (!PrintedMetadata) {
    # Ensure that we've printed the metadata and are now printing content
    print "Failed to print metadata before content" > "/dev/stderr"
    exit 1
  }
  print
}

END {
  print "]]>"
  print "</content>";
  print "</entry>";
}

function md5_to_uuid(md5sum) {
  return gensub(/(.{8})(.{4})(.{4})(.{4})(.{12})/,
                "\\1-\\2-\\3-\\4-\\5", "g", md5sum)
}
