#! /bin/gawk -f

# Example call:
# ./surround-content.awk -v template=../src/templates/foo.html content

# Available directives:
# #+CONTENT :: Allowed once per template
# #+EXEC <command> :: Command can use the "contentfile" keyword to refer to
#                     the content file

BEGIN {
  # Get filename manually because EXEC directives, which can occur during the
  # BEGIN section where the special FILENAME variable is not yet set, need it
  # for the "contentfile" keyword.
  filename = ARGV[ARGC - 1]

  tmp = "cat " template
  while ((tmp | getline) > 0) {
    if ($1 == "#+CONTENT") {
      break;
    } else if ($1 == "#+EXEC") {
      exec($0)
    } else {
      print
    }
  }
}

{
  print;
}

END {
  while ((tmp | getline) > 0) {
    if ($1 == "#+EXEC") {
      exec($0)
    } else {
      print;
    }
  }
}

function exec (directive) {
  exec_command = substr($0, index($0,$2))
  gsub("contentfile", filename, exec_command)
  while ((exec_command | getline) > 0) {
    print
  }
  close(exec_command)
}
